Unit LDASapi;
{ ========================================================================== }
{ =   Lynx Tecnologia Eletrnica Ltda.                                     = }
{ =   Projeto:   Drivers LynxDAS para o AqDados 7       Data: 14/mai/2002  = }
{ =   Mdulo:    LDASapi                                                   = }
{ =   Autor:     Lauro                                                     = }
{ =   Descrio: Acesso s primitivas da DLL LynxDAS                       = }
{ =   Verso:    1.0                                                       = }
{ ========================================================================== }
{$ALIGN OFF}  { Desliga opo de alinhamento da memria }

{-------------------------  Descrio da Interface  -------------------------}
Interface
  Uses Windows;


{ ===================================================== }
{ =     Declarao de tipos e variveis pblicas      = }
{ ===================================================== }

  Type  {----------------------------------- Tipo usado na LDAS_GetSamples --}
    TpAcqStatus = record
      ieStatus: word;         // Status da aquisio
      nSamples: int64;        // Nmero de amostragens aquisitadas por canal
      iLast   : int64;        // Nmero de amostragens lidas pela aplicao
      nSampGot: integer;      // Nmero de amostragens lidas na primitiva
    end;

  Type  {---------------------------------------- Faixas de entrada do A/D --}
    TpAiRange = array [0..15] of single;

  Const
    cBufSize = 16384;     { Tamanho do buffer em bytes }

  Type {-------------------------- Parmetro de sada da LDAS_GetSamplesVB --}
    TpUserBuf = array [0..cBufSize-1] of smallint;


{ ===================================================== }
{ =          Declarao de variveis pblicas         = }
{ ===================================================== }

  Var
    DrvVersion : byte;         // Verso do driver
    DrvRelease : byte;
    nAiChannels: smallint;     // Nmero de canais A/D
    nAiRange   : smallint;     // Nmero de faixas de entrada do A/D
    AiRange    : TpAiRange;    // Faixas de entrada do A/D
    nDiPorts   : smallint;     // Nmero de ports de entrada digital
    nBitsDi    : smallint;     // Nmero de bits por port de entrada digital
    nDoPorts   : smallint;     // Nmero de ports de sada digital
    nBitsDo    : smallint;     // Nmero de bits por port de sada digital


{ ========================================================================== }
{ =                      Primitivas do biblioteca DLL                      = }
{ ========================================================================== }

  {-- Primitivas de controle de acesso e iniciao --}
  Function  LDAS_OpenDriver (szDrvName: pchar): boolean; stdcall;
  Procedure LDAS_CloseDriver; stdcall;
  Function  LDAS_OpenAccess: boolean; stdcall;
  Procedure LDAS_CloseAccess; stdcall;
  Function  LDAS_GetVersion (Var Version, Release: byte): boolean; stdcall;

  Function  LDAS_GetAiCaps (Var nAiChannels, nAiRange: smallint;
                             Var AiRange: TpAiRange): boolean; stdcall;
  Function  LDAS_GetDioCaps (Var nDiPorts, nBitsDi: smallint;
                             Var nDoPorts, nBitsDo: smallint): boolean; stdcall;

  {-- Primitivas da aquisio de sinais --}
  Procedure LDAS_ClearCM; stdcall;
  Function  LDAS_InsertAiCM (Channel, iRange: smallint): boolean; stdcall;
  Function  LDAS_InsertDiCM (Group: smallint): boolean; stdcall;
  Function  LDAS_AcquisitionSetup (Var SampleFreq: single): boolean; stdcall;
  Function  LDAS_StartAcquisition: boolean; stdcall;
  Procedure LDAS_StopAcquisition; stdcall;
  Function  LDAS_GetSamples (Var Status: TpAcqStatus; Var Buffer; BufSize: integer): boolean; stdcall;
  Function  LDAS_GetSamplesVB (Var ieStatus, nSamples, nSampGot, iLast: integer;
                               Var UserBuf: TpUserBuf): boolean; stdcall;

  {-- Primitivas de E/S analgica --}
  Function  LDAS_ReadAi (Channel, iRange: smallint;
                         Var Value: smallint): boolean; stdcall;

  {-- Primitivas de E/S digital --}
  Function  LDAS_ReadDi (Group: smallint; Var Value: word): boolean; stdcall;
  Function  LDAS_WriteDo (Group: smallint; Value: word): boolean; stdcall;


{--------------------------  Bloco de Implementao  ------------------------}
Implementation


{ ========================================================================== }
{ =              Primitivas de controle de acesso ao driver                = }
{ ========================================================================== }

{-------------------------------------------------- Abre o acesso ao drive --}
Function LDAS_OpenDriver; stdcall; external 'LynxDAS.DLL';

{------------------------------------------------ Fecha o acesso ao driver --}
Procedure LDAS_CloseDriver; stdcall; external 'LynxDAS.DLL';

{--------------------------------------------- Inicia o acesso  placa A/D --}
Function LDAS_OpenAccess: boolean; stdcall; external 'LynxDAS.DLL';

{------------------------------------------- Finaliza o acesso  placa A/D --}
Procedure LDAS_CloseAccess; stdcall; external 'LynxDAS.DLL';

{----------------------------------------------- Informa averso do driver --}
Function LDAS_GetVersion; stdcall; external 'LynxDAS.DLL';

{-------------------- Informa os recursos de entrada analgica disponveis --}
Function LDAS_GetAiCaps; stdcall; external 'LynxDAS.DLL';

{-------------------------- Informa os recursos de E/S digital disponveis --}
Function LDAS_GetDioCaps; stdcall; external 'LynxDAS.DLL';

{----------------------------------------------- Limpa a memria de canais --}
Procedure LDAS_ClearCM; stdcall; external 'LynxDAS.DLL';

{--------------------------------------- Insere canal na memria de canais --}
Function LDAS_InsertAiCM; stdcall; external 'LynxDAS.DLL';

{----------------------------- Insere entrada digital na memria de canais --}
Function LDAS_InsertDiCM; stdcall; external 'LynxDAS.DLL';

{--------------------------------------- Preparao da aquisio de sinais --}
Function LDAS_AcquisitionSetup; stdcall; external 'LynxDAS.DLL';

{-------------------------------------------- Inicia a aquisio de sinais --}
Function LDAS_StartAcquisition; stdcall; external 'LynxDAS.DLL';

{------------------------------------------ Finaliza a aquisio de sinais --}
Procedure LDAS_StopAcquisition; stdcall; external 'LynxDAS.DLL';

{---------- Leitura das amostras aquisitadas pelo driver (no usar com VB) --}
Function LDAS_GetSamples; stdcall; external 'LynxDAS.DLL';

{---------- Leitura das amostras aquisitadas pelo driver (para uso com VB) --}
Function LDAS_GetSamplesVB; stdcall; external 'LynxDAS.DLL';

{----------------------------------- Leitura de canal de entrada analgica --}
Function LDAS_ReadAi; stdcall; external 'LynxDAS.DLL';

{-------------------------------------- Leitura de port de entrada digital --}
Function LDAS_ReadDi; stdcall; external 'LynxDAS.DLL';

{-------------------------------------- Escrita em port de entrada digital --}
Function LDAS_WriteDo; stdcall; external 'LynxDAS.DLL';


{ ===================================================== }
{ =            Rotina de iniciao do mdulo          = }
{ ===================================================== }

end.

