Unit TestIRQ;
{ ========================================================================== }
{ =   Lynx Tecnologia Eletronica Ltda.                                     = }
{ =   Projeto:   Drivers LynxDAS para o AqDados 7       Data: 14/mai/2002  = }
{ =   Programa:  Exemplo de uso do LynxDAS.DLL                             = }
{ =   Mdulo:    TestIRQ                                                   = }
{ =   Autor:     Lauro                                                     = }
{ =   Descrio: Teste da aquisicao por interrupo.                       = }
{ =   Verso:    1.0                                                       = }
{ ========================================================================== }
{
  So aquisitados os canais de 0 at nCanais-1. Os canais pares so
  programados na primeira faixa de entrada os impares na segunda faixa
  de entrada do A/D.
}

{$ALIGN OFF}

{-------------------------  Descrio da Interface  -------------------------}
Interface

Uses Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
     StdCtrls, Spin, Grids, ComCtrls, ExtCtrls, Buttons, LDASapi;


{ ===================================================== }
{ =       Declarao dos Tipos da Srie Temporal      = }
{ ===================================================== }

  Const NumEveMax = 32;

  Type
    TpDtHr = record
      Dia,Mes,Ano,HH,Mn,Sg,MS: word;
    end;
    FileNameType   = string [79];    { PathStr }
    SignalNameType = string [15];
    UnidadeType    = string [7];
    ComentarioType = string [30];

    WindowFFTType = (WdRetangular, WdHanning,  WdHamming,
                     WdTriangular, WdBlackman, WdKaiser);

    TipoAqType = (AqDMA, AqCPU, AqCepstrum, AqAntiCepstrum, AqCepsPot);

    OrigemSTType = (OrEnsaio, OrGeracao, OrCepstrum, OrAntiCepstrum,
                    OrCombArq1, OrCombArq2, OrCombArq3, OrCombArq4,
                    OrUniSer, OrUniPar, OrOpeAri, OrSel, OrMed, OrFiltro,
                    OrOpMFT, OrCepsPot, OrRainMult, OrPolar);

  Type
    ConfCanaisType = record
      Nome   : SignalNameType;   { Denominao do canal }
      Unidade: UnidadeType;      { Unidade do canal }
      LimMax : single;           { Lim. mximo da escala }
      LimMin : single;           { Lim. mnimo da escala }
      Usado  : boolean;          { Indica se ativo }
      Janela : word              { Indica Tamanho Janela }
    end;

    CanaisEntrada = array [0..31] of ConfCanaisType;
    EventosType   = array [1..NumEveMax] of longint;

    HeaderTempType = record
      DtHr      : TpDtHr;           { Data e hora do incio do ensaio }
      Comentario: ComentarioType;   { Comentrio do ensaio }
      CnAD      : CanaisEntrada;    { Configurao dos canais }
      NoCnAtivos: smallint;         { Nmero de canais ativos }
      RangeAD   : longint;          { Range do conversor A/D (ex. 1024) }
      FreqAmostr: single;           { Frequncia de amostragem }
      NoAmostras: longint;          { Nmero de amostras por canal }
      TipoAq    : TipoAqType;       { Tipo de aquisio: DMA ou CPU }
      Origem    : OrigemSTType;     { Origem da srie: Aquis., etc. }
      ArqOrigem1: FileNameType;     { Nome do arquivo de origem 1 }
      ComOrigem1: ComentarioType;   { Comentrio do arquivo de origem 1 }
      DtHr1     : TpDtHr;           { Data e hora do arq. origem 1 }
      ArqOrigem2: FileNameType;     { Nome do arquivo de origem 2 }
      ComOrigem2: ComentarioType;   { Comentrio do arquivo de origem 2 }
      DtHr2     : TpDtHr;           { Data e hora do arq. origem 2 }
      ExCom     : array [1..6] of
                    ComentarioType; { Outros comentrios }
      NoEventos : smallint;         { Nmero de eventos (AqDados) }
      Eventos   : EventosType;      { Tabela de eventos (instantes) }
    end;


{ ===================================================== }
{ =               Declarao de tipos                 = }
{ ===================================================== }

  Type
    TpCoefReta = array [0..1] of single;

  Type TdlgTestIRQ = class(TForm)
    gbParametros: TGroupBox;
    cbFreqAq    : TComboBox;
    cbNoAmost   : TComboBox;
    seNoCanais  : TSpinEdit;
    lblFreqAq   : TLabel;
    lblNoAmost  : TLabel;
    lblNoCanais : TLabel;
    pbAquisicao : TProgressBar;
    sgTabela    : TStringGrid;
    StatusBar   : TStatusBar;
    tmrUpdate   : TTimer;
    ckGravaST   : TCheckBox;
    panToolBar  : TPanel;
    btStart     : TBitBtn;
    btStop      : TBitBtn;
    btClose     : TBitBtn;
    Procedure FormCreate (Sender: TObject);
    Procedure FormClose (Sender: TObject; var Action: TCloseAction);
    Procedure btStartClick (Sender: TObject);
    Procedure btStopClick (Sender: TObject);
    Procedure seNoCanaisChange (Sender: TObject);
    procedure tmrUpdateTimer(Sender: TObject);
  private
    { Private declarations }
    fAtivo    : boolean;       { Indica ensaio ativo }
    FreqAq    : single;        { Frequncia de amostragem }
    nCanais   : word;          { Nmero de canais a aquisitar }
    nAmostras : longint;       { Nmero de amostras por canal }
    nAmsBuf   : word;          { Nmero de amostras por buffer por canal }
    vAng      : TpCoefReta;    { Coeficiente angular p/ conversar em volts }
    vLin      : TpCoefReta;    { Coeficiente linear p/ conversar em volts }

    AcqStatus : TpAcqStatus;   { Status da aquisi0 }
    iAmsEns   : longint;       { Nmero da amostra no ensaio }
    inTab     : longint;       { Limite (em ams) para atualizao da tabela }
    idTab     : longint;       { Intervalo (em ams) para atualizao da tabela }
    hdST      : HeaderTempType;{ Cabealho da srie temporal }
    aST       : file;          { Arquivo de srie temporal }
    fGravaST  : boolean;       { Indica se grava a serie temporal }
    TamBuf    : longint;       { Tamanho do buffer para gravao }

    Procedure EnableControls;
    Procedure FinalizaAquisicao;
    Procedure GravaCabecalho;
    Procedure IniciaAquisicao;
    Procedure MontaCabecalho;
    Procedure ProgramaCanais;
    Procedure UpdateTabela;
    Procedure UpdateTabelaFixa;
  public
    { Public declarations }
    nBuffers  : smallint;      { Nmero de buffers de aquisio alocados }
  end;


{ ===================================================== }
{ =                Variveis globais                  = }
{ ===================================================== }

  Var dlgTestIRQ: TdlgTestIRQ;


{-------------------------  Descrio da Implementao ----------------------}
Implementation

{$R *.DFM}

{ ===================================================== }
{ =           Declarao de variveis locais          = }
{ ===================================================== }

  Const {--------------------- Tabela de frequncias de amostragem (em Hz) --}
    cTabFreq: array [0..10] of single =
      (50, 60, 100, 120, 200, 240, 300, 500, 1000, 2000, 5000);

  Const {-------------- Tabela de nmero de amostras a aquisitar por canal --}
    cTabAms: array [0..6] of longint =
      (0, 1000, 2000, 5000, 10000, 20000, 50000);

   Var {-------------------------- Informao da aquisio por interrupo --}
     UserBuf: TpUserBuf;


{ ========================================================================== }
{ =                  Implementao do objeto TdlgTestIRQ                   = }
{ ========================================================================== }

{-------------------------------- TdlgTestIRQ: Trata criao do formulrio --}
Procedure TdlgTestIRQ.FormCreate (Sender: TObject);
  Var i   : smallint;
      sAux: string;
begin
  {-- Inicia combo box da frequncia de amostragem --}
  cbFreqAq.Clear;
  for  i := 0 to 12  do
  begin
    sAux := Format ('%.6g Hz', [cTabFreq [i]]);
    cbFreqAq.Items.Add (sAux);
  end;
  {-- Inicia combo box do nmero de amostras --}
  cbNoAmost.Clear;
  cbNoAmost.Items.Add ('2 bilhes');
  for  i := 1 to 6  do
    cbNoAmost.Items.Add (IntToStr (cTabAms [i]));

  cbFreqAq.ItemIndex    := 5;       { 240 Hz }
  cbNoAmost.ItemIndex   := 3;    { 5000 amostras }
  seNoCanais.Value      := 4;
  sgTabela.Cells [0, 0] := 'A/D';
  sgTabela.Cells [1, 0] := 'Faixa';
  sgTabela.Cells [2, 0] := 'Tenso (V)';
  StatusBar.Panels.Items [0].Text := '';
  StatusBar.Panels.Items [1].Text := '';
  fAtivo := false;
  EnableControls;
  UpdateTabelaFixa;
  {-- Coeficientes da converso dos canais configurados --}
  {-- para a primeira faixa de entrada --}
  if  AiRange [0] < 0   {  bipolar ? }
  then begin
    vAng [0] := - AiRange [0] / 32768.0;
    vLin [0] := 0;
  end
  else begin
    vAng [0] := AiRange [0] / 65536.0;
    vLin [0] := AiRange [0] / 2;
  end;
  {-- Coeficientes da converso dos canais configurados --}
  {-- para a segunda faixa de entrada --}
  if  AiRange [1] < 0   {  bipolar ? }
  then begin
    vAng [1] := - AiRange [1] / 32768.0;
    vLin [1] := 0;
  end
  else begin
    vAng [1] := AiRange [1] / 65536.0;
    vLin [1] := AiRange [1] / 2;
  end;
end;

{----------------------- TdlgTestIRQ: Trata fechamento da caixa de dilogo --}
Procedure TdlgTestIRQ.FormClose (Sender: TObject; var Action: TCloseAction);
begin
  if  fAtivo  then
    FinalizaAquisicao;
end;

{----------------------- TdlgTestIRQ: Trata boto para iniciar a aquisio --}
Procedure TdlgTestIRQ.btStartClick (Sender: TObject);
begin
  if  not fAtivo
  then begin
    pbAquisicao.Position            := 0;
    StatusBar.Panels.Items [0].Text := '';
    StatusBar.Panels.Items [1].Text := 'Aquisitando...';
    fAtivo := true;
    EnableControls;
    UpdateTabelaFixa;
    IniciaAquisicao;
  end;
end;

{------------------- TdlgTestIRQ: Trata boto para interromper a aquisio --}
Procedure TdlgTestIRQ.btStopClick (Sender: TObject);
begin
  if  fAtivo
  then begin
    FinalizaAquisicao;
    StatusBar.Panels.Items [1].Text := 'Aquisio interrompida manualmente.';
  end;
end;

{------------------------- TdlgTestIRQ: Atualiza habilitao dos controles --}
Procedure TdlgTestIRQ.EnableControls;
begin
  btStart.Enabled    := not fAtivo;
  btStop.Enabled     := fAtivo;
  cbFreqAq.Enabled   := not fAtivo;
  cbNoAmost.Enabled  := not fAtivo;
  seNoCanais.Enabled := not fAtivo;
  tmrUpdate.Enabled  := fAtivo;
end;

{----------------------- TdlgTestIRQ: Finaliza a aquisio por interrupo --}
Procedure TdlgTestIRQ.FinalizaAquisicao;
begin
  LDAS_StopAcquisition;     { Encerra aquisio por interrupo }
  fAtivo := false;
  EnableControls;
  if  fGravaST
  then begin
    hdST.NoAmostras := iAmsEns;
    Seek (aST, 0);
    BlockWrite (aST, hdST, SizeOf (hdST));
    CloseFile (aST);
  end;
end;

{-------------------------- TdlgTestIRQ: Grava cabealho da srie temporal --}
Procedure TdlgTestIRQ.GravaCabecalho;
begin
  if  not fGravaST  then exit;
  AssignFile (aST, 'TESTE.TEM');
  Rewrite (aST, 1);
  BlockWrite (aST, hdST, SizeOf (hdST));
  TamBuf := nAmsBuf * 2 * nCanais;
end;

{------------------------- TdlgTestIRQ: Inicia a aquisio por interrupo --}
Procedure TdlgTestIRQ.IniciaAquisicao;
begin
  nCanais    := seNoCanais.Value;               { Nmero de canais a aquisitar }
  nAmostras  := cTabAms [cbNoAmost.ItemIndex];
  FreqAq     := cTabFreq [cbFreqAq.ItemIndex];  { Frequncia de amostragem por canal }
  fGravaST   := ckGravaST.Checked;
  ProgramaCanais;
  iAmsEns    := 0;
  idTab      := Round (FreqAq / 2);             { Atualiza tabela 2 vezes por segundo }
  inTab      := idTab;                          { Limite (em ams) para atualizao da tabela }

  {-- Inicia arquivo de srie temporal (AqDados) --}
  MontaCabecalho;
  GravaCabecalho;

  {-- Programa a aquisio  --}
  if  LDAS_AcquisitionSetup (FreqAq)  then
     LDAS_StartAcquisition;   {-- Dispara a aquisio  --}
end;

{------------------------ TdlgTestIRQ: Monta o cabealho da srie temporal --}
Procedure TdlgTestIRQ.MontaCabecalho;
  Procedure SetCnAD (Cn: smallint; Sup, Inf: single);
  begin
      hdST.CnAD [Cn].Usado  := true;
      hdST.CnAD [Cn].LimMax := Sup;
      hdST.CnAD [Cn].LimMin := Inf;
    end;
  Var i : smallint;
      dt: TDateTime;
      Range: single;
begin
  with  hdST  do
  begin
    dt := Now;    { Data/hora corrente }
    with  DtHr  do
    begin
      DecodeDate (dt, Ano, Mes, Dia);
      DecodeTime (dt, HH, Mn, Sg, MS);
    end;
    Comentario := 'VxD Test';
    for  i := 0 to 31  do with  CnAD [i]  do
    begin
      Nome    := 'Canal ' + IntToStr (i);
      Unidade := 'volts';
      LimMax  := 5;
      LimMin  := -5;
      Usado   := false;
      Janela  := 0;
    end;
    NoCnAtivos := nCanais;
    RangeAD    := 4096;
    FreqAmostr := FreqAq;
    NoAmostras := nAmostras;
    TipoAq     := AqCPU;
    Origem     := OrEnsaio;
    ArqOrigem1 := '';
    ComOrigem1 := '';
    ArqOrigem2 := '';
    ComOrigem2 := '';
    DtHr1      := DtHr;
    DtHr2      := DtHr;
    for  i := 1 to 6  do ExCom [i] := Comentario;
    NoEventos := 0;
  end;
  for  i := 0 to Pred (nCanais)  do
  begin
    if  (i mod 2) = 0  then
      Range := AiRAnge [0]  { Primeira faixa de entrada }
    else
      Range := AiRAnge [1]; { Segunda faixa de entrada }
    if  Range < 0  then  { bipolar ? }
      SetCnAD (i, -Range, Range)
    else
      SetCnAD (i, -Range, 0);
  end;
end;

{--------------------------- TdlgTestIRQ: Programa a memria de canais A/D --}
{ neste exemplo a posio na memria de canais corresponde
  ao nmero do canal A/D
}
Procedure TdlgTestIRQ.ProgramaCanais;
  Var Canal: byte;
begin
  LDAS_ClearCM;                       { Limpa a memria e canais }
  for  Canal := 0 to Pred (nCanais)  do
    if  (Canal mod 2) = 0  then
      LDAS_InsertAiCM (Canal, 0)    { Primeira faixa de entrada }
    else
      LDAS_InsertAiCM (Canal, 1);   { Segunda faixa de entrada }
end;

{-------------------------- TdlgTestIRQ: Trata mudana do nmero de canais --}
Procedure TdlgTestIRQ.seNoCanaisChange (Sender: TObject);
begin
  UpdateTabelaFixa;
end;

{-------------------------- TdlgTestIRQ: Trata temporizao de atualizao --}
Procedure TdlgTestIRQ.tmrUpdateTimer (Sender: TObject);
  Var Size: longint;
begin
  if  fAtivo
  then begin
    if  not LDAS_GetSamples (AcqStatus, UserBuf, SizeOf (UserBuf))  then
      StatusBar.Panels.Items [1].Text := 'Aquisio no est ativa!'
    else begin
      iAmsEns  := AcqStatus.iLast;
      if  (AcqStatus.ieStatus <> 0) or (iAmsEns >= nAmostras) and (nAmostras <> 0)
      then begin
        if  AcqStatus.ieStatus <> 0  then
          StatusBar.Panels.Items [1].Text := 'Erro na aquisio!'
        else begin
          if  fGravaST and (AcqStatus.nSampGot > 0)
          then begin
            Size := nCanais * 2 * AcqStatus.nSampGot;
            BlockWrite (aST, UserBuf, Size);
          end;
          StatusBar.Panels.Items [1].Text := 'Fim da Aquisio.';
        end;
        FinalizaAquisicao;
      end
      else begin
        if  (AcqStatus.nSampGot > 0)
        then begin
          StatusBar.Panels.Items [0].Text := Format ('Amostra: %d', [iAmsEns]);
          if  nAmostras > 0  then
            pbAquisicao.Position := Round (100 * iAmsEns / nAmostras);
          UpdateTabela;
          Inc (inTab, idTab);
        end;
        if  fGravaST and (AcqStatus.nSampGot > 0)
        then begin
          Size := nCanais * 2 * AcqStatus.nSampGot;
          BlockWrite (aST, UserBuf, Size);
        end;
      end;
    end;
  end;
end;

{--------------- TdlgTestIRQ: Atualiza a tabela com ltimo dado aquisitado --}
Procedure TdlgTestIRQ.UpdateTabela;
  Var Canal : smallint;
      ic    : smallint;
      vVolts: single;
begin
  for  Canal := 0 to Pred (nCanais)  do
  begin
    ic     := Canal mod 2;
    vVolts := UserBuf [Canal] * vAng [ic] + vLin [ic];
    sgTabela.Cells [2, Canal+1] := Format ('%8.3f', [vVolts]);
  end;
end;

{------------------------------------- TdlgTestIRQ: Atualiza a tabela fixa --}
Procedure TdlgTestIRQ.UpdateTabelaFixa;
  Var Canal: word;
      Range: single;
begin
  sgTabela.RowCount := seNoCanais.Value + 1;
  for  Canal := 0 to Pred (seNoCanais.Value)  do
  begin
    sgTabela.Cells [0, Canal+1] := Format (' %.2d', [Canal]);
    sgTabela.Cells [2, Canal+1] := '';
    if  (Canal mod 2) = 0  then
      Range := AiRAnge [0]  { Primeira faixa de entrada }
    else
      Range := AiRAnge [1]; { Segunda faixa de entrada }

    if  Range < 0  then  {  bipolar ? }
      sgTabela.Cells [1, Canal+1] := Format ('+/- %.5g V', [-Range])
    else
      sgTabela.Cells [1, Canal+1] := Format ('0 a %.5g V', [Range])
  end;
end;

end.
